﻿using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.Json;
using System.Text.Json.Serialization;
using InnoLabProjektDektopApp.Services;
using System.Diagnostics;
using static InnoLabProjektDektopApp.DistractionsList;
using System;
using InnoLabProjektDektopApp.Utils;


namespace InnoLabProjektDektopApp
{
    public partial class ProgramsList : Page
    {
        private Dictionary<string, List<ProcessEntry>> _data;
        private readonly string _jsonFilePath;

        public ProgramsList()
        {
            InitializeComponent();
            _jsonFilePath = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Assets", "distractingPrograms.json");
            LoadProgramList();
            RefreshRunningProcessList();
        }

        private void SearchBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            // Platzhalter ein- oder ausblenden
            if (!string.IsNullOrEmpty(SearchBox.Text))
            {
                SearchPlaceholder.Visibility = Visibility.Collapsed; // Ausblenden
            }
            else
            {
                SearchPlaceholder.Visibility = Visibility.Visible; // Einblenden
            }

            // Filterlogik für die Suchergebnisse
            string searchText = SearchBox.Text?.ToLower() ?? string.Empty;

            // Clear current UI
            ItemsPanel.Children.Clear();

            // Filter items based on search text
            if (_data != null && _data.TryGetValue("Programs", out List<ProcessEntry>? value))
            {
                var filteredItems = value.Where(
                    item => string.IsNullOrEmpty(searchText) || 
                    item.ProcessName.Contains(searchText, StringComparison.CurrentCultureIgnoreCase) ||
                    item.MainWindowTitle.Contains(searchText, StringComparison.CurrentCultureIgnoreCase)
                    )
                    .ToList();

                foreach (var item in filteredItems)
                {
                    // Create a StackPanel for each filtered item
                    ShowNewItemInList(item.ProcessName, item.MainWindowTitle, item.IsDistracting);
                }
            }
            RefreshRunningProcessList(searchText);
        }


        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            RefreshRunningProcessList();
        }

        private void RefreshRunningProcessList(string filter = null)
        {
            var windowedProcesses = ProcessMonitor.GetWindowedProcesses();
            windowedProcesses = windowedProcesses.Where(
                    item => string.IsNullOrEmpty(filter) ||
                    item.Name.Contains(filter, StringComparison.CurrentCultureIgnoreCase) ||
                    item.MainWindowTitle.Contains(filter, StringComparison.CurrentCultureIgnoreCase)
                    )
                    .ToList();

            ItemsPanel2.Children.Clear();

            foreach (var process in windowedProcesses)
            {
                // Prüfen, ob der Prozess bereits in der Liste ist
                if (_data["Programs"].Any(p => p.ProcessName.Equals(process.Name, StringComparison.CurrentCultureIgnoreCase)))
                    continue;

                // Grid für bessere Platzverteilung
                var newGrid = new Grid { Margin = new Thickness(5, 2, 5, 2) };

                // Spalten:
                // 1. Dynamische Spalte für den TextBlock
                // 2. Feste Spalte für den Button
                newGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
                newGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = GridLength.Auto });

                // TextBlock für Prozessname mit TextWrapping
                var textBlock = new TextBlock
                {
                    Text = $"{process.MainWindowTitle} ({process.Name})",
                    Margin = new Thickness(5),
                    ToolTip = $"{process.MainWindowTitle} ({process.Name})",
                    TextWrapping = TextWrapping.Wrap
                };

                // Button als Border mit Style "RoundedButtonBorder"
                var addButtonBorder = new Border
                {
                    Style = (Style)Application.Current.Resources["RoundedButtonBorder"],
                    Child = new TextBlock
                    {
                        Text = "+",
                        Margin = new Thickness(0, 0, 0, 4),
                        HorizontalAlignment = HorizontalAlignment.Center,
                        VerticalAlignment = VerticalAlignment.Center,
                        Foreground = (Brush)Application.Current.Resources["BACKGROUND_PRIMARY_BRUSH"],
                        FontSize = 24,
                        FontWeight = FontWeights.Bold
                    },
                    Margin = new Thickness(5, 0, 10, 0), // Rechts Abstand zur Scrollbar/Rand
                    Cursor = Cursors.Hand,
                    Width = 30,  // Feste Breite
                    Height = 30  // Feste Höhe
                };

                // Klick-Event für Button
                addButtonBorder.MouseDown += (s, args) =>
                {
                    AddProcessFromList(process.Name, process.MainWindowTitle);
                };

                // Elemente ins Grid setzen
                Grid.SetColumn(textBlock, 0);  // Dynamische Spalte für Text
                Grid.SetColumn(addButtonBorder, 1); // Button bleibt konstant rechts

                newGrid.Children.Add(textBlock);
                newGrid.Children.Add(addButtonBorder);

                // Zum UI-Stack hinzufügen
                ItemsPanel2.Children.Add(newGrid);
            }
        }


        private void AddProcessFromList(string processName, string mainWindowTitle)
        {
            // Check if the process is already in the list
            if (_data.TryGetValue("Programs", out List<ProcessEntry>? value) && value.Any(p => p.ProcessName.Equals(processName, StringComparison.CurrentCultureIgnoreCase)))
            {
                MessageBox.Show($"Process '{processName}' is already added.", "Info", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }

            //show a message box to give the user the option to change the main window title
            var inputDialog = new InputDialog("Enter a new Title:", mainWindowTitle); // Assuming InputDialog is a custom dialog
            if (inputDialog.ShowDialog() == true)
            {
                mainWindowTitle = inputDialog.Input; // Get updated title
            }

            if (inputDialog.DialogResult == false) return;


            // Add the process to the list
            if (!_data.ContainsKey("Programs"))
            {
                _data["Programs"] = [];
            }
            _data["Programs"].Add(new ProcessEntry
            {
                ProcessName = processName.ToLower(),
                MainWindowTitle = mainWindowTitle,
                IsDistracting = true
            });

            ShowNewItemInList(processName.ToLower(), mainWindowTitle, true);
            RefreshRunningProcessList();

            // Save changes to the JSON
            SaveData();
        }

        private void ShowNewItemInList(string processName, string mainWindowTitle, bool isDistracting)
        {
            // Add to UI dynamically
            var newGrid = new Grid { Margin = new Thickness(5, 2, 5, 2) };

            // Drei Spalten: 1. für CheckBox + Text, 2. für den Edit Button, 3. für den Delete Button
            newGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            newGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = GridLength.Auto });
            newGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = GridLength.Auto });

            var newCheckBox = new CheckBox
            {
                Margin = new Thickness(5),
                IsChecked = isDistracting,
                ToolTip = $"{mainWindowTitle} ({processName})",
                Tag = new ProcessEntry { ProcessName = processName, MainWindowTitle = mainWindowTitle, IsDistracting = isDistracting }
            };

            var textBlock = new TextBlock
            {
                Text = $"{mainWindowTitle} ({processName})",
                TextWrapping = TextWrapping.Wrap,
            };
            newCheckBox.Content = textBlock;

            newCheckBox.Checked += CheckBox_CheckedChanged;
            newCheckBox.Unchecked += CheckBox_CheckedChanged;

            var editButtonBorder = new Border
            {
                Style = (Style)Application.Current.Resources["RoundedButtonBorder"],
                Child = new TextBlock
                {
                    Text = "Edit",
                    HorizontalAlignment = HorizontalAlignment.Center,
                    VerticalAlignment = VerticalAlignment.Center,
                    Foreground = (Brush)Application.Current.Resources["BACKGROUND_PRIMARY_BRUSH"],
                    FontWeight = FontWeights.Bold
                },
                Margin = new Thickness(5, 0, 10, 0),
                Cursor = Cursors.Hand,
                Width = 50,
                Height = 30,
                CornerRadius = new CornerRadius(10)
            };
            editButtonBorder.MouseDown += (s, e) => EditItemInList(processName, mainWindowTitle);

            var deleteButtonBorder = new Border
            {
                Style = (Style)Application.Current.Resources["RoundedButtonBorder"],
                Child = new TextBlock
                {
                    Text = "Delete",
                    HorizontalAlignment = HorizontalAlignment.Center,
                    VerticalAlignment = VerticalAlignment.Center,
                    Foreground = (Brush)Application.Current.Resources["BACKGROUND_PRIMARY_BRUSH"],
                    FontWeight = FontWeights.Bold
                },
                Margin = new Thickness(5, 0, 10, 0),
                Cursor = Cursors.Hand,
                Width = 50,
                Height = 30,
                CornerRadius = new CornerRadius(10)
            };
            deleteButtonBorder.MouseDown += (s, e) => DeleteItemInList(processName);

            // Set elements in the Grid
            Grid.SetColumn(newCheckBox, 0);
            Grid.SetColumn(editButtonBorder, 1);
            Grid.SetColumn(deleteButtonBorder, 2);

            newGrid.Children.Add(newCheckBox);
            newGrid.Children.Add(editButtonBorder);
            newGrid.Children.Add(deleteButtonBorder);

            // Add element to the UI stack
            ItemsPanel.Children.Add(newGrid);
        }

        private void EditItemInList(string processName, string mainWindowTitle)
        {
            var inputDialog = new InputDialog("Enter a new Title:", mainWindowTitle);
            inputDialog.ShowDialog();
            if (inputDialog.DialogResult == false || inputDialog.Input.Length == 0) return;

            _data["Programs"].FirstOrDefault(p => p.ProcessName.Equals(processName, StringComparison.CurrentCultureIgnoreCase))!.MainWindowTitle = inputDialog.Input;

            SaveData();
            RefreshProgramList();
        }

        private void DeleteItemInList(string processName)
        {
            _data["Programs"].RemoveAll(p => p.ProcessName.Equals(processName, StringComparison.CurrentCultureIgnoreCase));
            SaveData();
            RefreshProgramList();
        }

        public void RefreshProgramList()
        {
            ItemsPanel.Children.Clear();

            // Kategorie-Elemente laden
            var items = _data["Programs"];
            foreach (var item in items)
            {
                ShowNewItemInList(item.ProcessName, item.MainWindowTitle, item.IsDistracting);
            }

            // Überschrift setzen
            HeaderTextBlock.Text = "Programs";
        }

        public void LoadProgramList()
        {
            try
            {
                // JSON-Datei lesen
                string jsonContent = File.ReadAllText(_jsonFilePath);

                // JSON deserialisieren
                _data = JsonSerializer.Deserialize<Dictionary<string, List<ProcessEntry>>>(jsonContent);

                if (_data != null && _data.ContainsKey("Programs"))
                {
                    // Kategorie-Elemente laden
                    var items = _data["Programs"];
                    foreach (var item in items)
                    {
                        ShowNewItemInList(item.ProcessName, item.MainWindowTitle, item.IsDistracting);
                    }

                    // Überschrift setzen
                    HeaderTextBlock.Text = "Programs";
                }
                else
                {
                    MessageBox.Show($"No items found for category 'Programs'.");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error loading items: {ex.Message}");
            }
        }


        private void CheckBox_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (sender is CheckBox checkBox && checkBox.Tag is ProcessEntry item)
            {
                // Wert in der JSON-Datenstruktur aktualisieren
                item.IsDistracting = checkBox.IsChecked ?? false;

                // Write the value into the _data dictionary
                _data["Programs"].FirstOrDefault(p => p.ProcessName.Equals(item.ProcessName, StringComparison.CurrentCultureIgnoreCase))!.IsDistracting = item.IsDistracting;

                // Änderungen in die Datei schreiben
                SaveData();
            }
        }

        private void SaveData()
        {
            try
            {
                string updatedJson = JsonSerializer.Serialize(_data, new JsonSerializerOptions { WriteIndented = true });
                File.WriteAllText(_jsonFilePath, updatedJson);
                MessageBox.Show("Data saved successfully!");
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error saving data: {ex.Message}");
            }
        }
        public class ProcessEntry
        {
            [JsonPropertyName("processName")]
            public required string ProcessName { get; set; }

            [JsonPropertyName("mainWindowTitle")]
            public required string MainWindowTitle { get; set; }

            [JsonPropertyName("distracting")]
            public bool IsDistracting { get; set; }
        }
    }
}