﻿using InnoLabProjektDektopApp.Services;
using LiveChartsCore;
using LiveChartsCore.SkiaSharpView;
using LiveChartsCore.SkiaSharpView.Painting;
using SkiaSharp;
using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace InnoLabProjektDektopApp.Screens.Regulaer
{
    /// <summary>
    /// Interaction logic for SessionStatistics.xaml
    /// </summary>
    public partial class SessionStatistics : Page
    {
        TimeSpan distractedTotal;
        TimeSpan breakTotal = new();
        TimeSpan pausedTotal = new();
        TimeSpan productiveTotal;
        int final_stage = 0;
        readonly int offset = 130;

        public IEnumerable<ISeries> Series { get; set; } = [];

        public event PropertyChangedEventHandler? PropertyChanged;

        private BitmapImage _centerImageSource = new();
        public BitmapImage CenterImageSource
        {
            get => _centerImageSource;
            set
            {
                _centerImageSource = value;
                OnPropertyChanged(nameof(CenterImageSource));
            }
        }

        protected void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public SessionStatistics(string sessionDataJsonPath)
        {
            DataContext = this;
            InitializeComponent();
            DrawChart(sessionDataJsonPath);
            UpdateSeriesValues();
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Overview());
        }

        private void DrawChart(string json_file_path)
        {
            // Example JSON data
            var jsonData = File.ReadAllText(json_file_path);

            // Deserialize JSON data
            var sessionData = System.Text.Json.JsonSerializer.Deserialize<SessionData>(jsonData);

            // Create a canvas to draw the chart
            int bar_width = 400;
            int bar_height = 20;

            if (sessionData == null)
            {
                return;
            }
            final_stage = sessionData.FinalDistractionStage;

            // Calculate the height of the rectangle
            int rectangleHeight = 190 + (int)(1.5 * bar_height * (sessionData.CycleStartTimes.Count-1));
            chartBackground.Height = rectangleHeight;

            // Draw bars for each cycle
            foreach (var cycle in sessionData.CycleStartTimes)
            {
                DateTime cycleStartTime = DateTime.Parse(cycle.Value);
                DateTime cycleEndTime = // start of next cycle or end of session
                    cycle.Key < sessionData.CycleStartTimes.Count
                    ? DateTime.Parse(sessionData.CycleStartTimes[cycle.Key + 1])
                    : DateTime.Parse(sessionData.SessionEndTime);

                // Calculate total cycle duration
                TimeSpan cycleDuration = cycleEndTime - cycleStartTime;

                // Draw the entire cycle in green
                Rectangle cycleSegment = new()
                {
                    Width = bar_width,
                    Height = bar_height,
                    Fill = Brushes.Green
                };

                Canvas.SetLeft(cycleSegment, 0);
                Canvas.SetTop(cycleSegment, (cycle.Key - 1) * 1.5 * bar_height + offset);
                chartCanvas.Children.Add(cycleSegment);

                // Add cycle number text
                TextBlock cycleText = new()
                {
                    Text = $"Cycle {cycle.Key}",
                    Foreground = Brushes.White,
                    FontSize = 15,
                    FontWeight = FontWeight.FromOpenTypeWeight(500)
                };

                Canvas.SetLeft(cycleText, bar_width + 5); // Adjust the position as needed
                Canvas.SetTop(cycleText, (cycle.Key - 1) * 1.5 * bar_height + offset);
                chartCanvas.Children.Add(cycleText);

                // Draw distracted segments in red
                foreach (var process in sessionData.ProcessInfoList.Where(p => p.Cycle == cycle.Key))
                {
                    DateTime processStartTime = DateTime.Parse(process.StartTime);
                    DateTime processEndTime = DateTime.Parse(process.EndTime);

                    // Calculate segment duration
                    TimeSpan segmentDuration = processEndTime - processStartTime;

                    // Determine if the process is a distraction

                    // Draw segment
                    Rectangle segment = new()
                    {
                        Width = bar_width * (segmentDuration / cycleDuration),
                        Height = bar_height,
                        Fill = Brushes.Red
                    };

                    Canvas.SetLeft(segment, bar_width * (processStartTime - cycleStartTime).TotalSeconds / cycleDuration.TotalSeconds);
                    Canvas.SetTop(segment, (cycle.Key - 1) * 1.5 * bar_height + offset);
                    chartCanvas.Children.Add(segment);
                }

                // Draw break segments in blue
                foreach (var breakInfo in sessionData.BreakInfoList.Where(b => b.Cycle == cycle.Key))
                {
                    DateTime breakStartTime = DateTime.Parse(breakInfo.StartTime);
                    DateTime breakEndTime = DateTime.Parse(breakInfo.EndTime);

                    // Calculate break duration
                    TimeSpan breakDuration = breakEndTime - breakStartTime;

                    // Draw break segment
                    Rectangle breakSegment;
                    if (breakInfo.IsBreak)
                    {
                        breakTotal += breakEndTime - breakStartTime;
                        breakSegment = new()
                        {
                            Width = bar_width * (breakDuration.TotalSeconds / cycleDuration.TotalSeconds),
                            Height = bar_height,
                            Fill = Brushes.Blue
                        };
                    }
                    else
                    {
                        pausedTotal += breakEndTime - breakStartTime;
                        breakSegment = new()
                        {
                            Width = bar_width * (breakDuration.TotalSeconds / cycleDuration.TotalSeconds),
                            Height = bar_height,
                            Fill = Brushes.LightBlue
                        };
                    }

                    Canvas.SetLeft(breakSegment, bar_width * (breakStartTime - cycleStartTime).TotalSeconds / cycleDuration.TotalSeconds);
                    Canvas.SetTop(breakSegment, (cycle.Key - 1) * 1.5 * bar_height + offset);
                    chartCanvas.Children.Add(breakSegment);
                }

            }
            productiveTotal = DateTime.Parse(sessionData.SessionEndTime) - (DateTime.Parse(sessionData.CycleStartTimes[1]) + breakTotal + pausedTotal + distractedTotal);
            // Add legend
            AddLegend();

            // Add axis
            Line yAxis = new()
            {
                X1 = 0,
                Y1 = offset - 5,
                X2 = 0,
                Y2 = 1.5 * 20 * (sessionData.CycleStartTimes.Count) + offset - 5,
                Stroke = Brushes.White,
                StrokeThickness = 2
            };
            Canvas.SetTop(yAxis, 0);
            Canvas.SetLeft(yAxis, 0);
            chartCanvas.Children.Add(yAxis);

            // Convert ProcessInfoList to List<(DateTime Start, DateTime End)>
            var processIntervals = sessionData.ProcessInfoList
                .Select(p => (Start: DateTime.Parse(p.StartTime), End: DateTime.Parse(p.EndTime)))
                .ToList();
            var allBreaksIntervals = sessionData.BreakInfoList
                .Select(p => (Start: DateTime.Parse(p.StartTime), End: DateTime.Parse(p.EndTime)))
                .ToList();
            var pauseIntervals = sessionData.BreakInfoList
                .Where(p => !p.IsBreak)
                .Select(p => (Start: DateTime.Parse(p.StartTime), End: DateTime.Parse(p.EndTime)))
                .ToList();
            var breakIntervals = sessionData.BreakInfoList
                .Where(p => p.IsBreak)
                .Select(p => (Start: DateTime.Parse(p.StartTime), End: DateTime.Parse(p.EndTime)))
                .ToList();

            // Merge intervals
            var mergedDistractionIntervals = ProcessMonitor.MergeIntervals(processIntervals);
            distractedTotal = ProcessMonitor.SubtractBreaksFromDistractionTime(mergedDistractionIntervals, allBreaksIntervals);
            breakTotal = breakIntervals.Aggregate(TimeSpan.Zero, (total, interval) => total + (interval.End - interval.Start));
        }

        private void AddLegend()
        {
            // Create legend items
            var legendItems = new List<(string Text, Brush Color)>
                    {
                        ("Productive Work", Brushes.Green),
                        ("Distraction", Brushes.Red),
                        ("Break", Brushes.Blue),
                        ("Pause", Brushes.LightBlue)
                    };

            // Draw legend
            for (int i = 0; i < legendItems.Count; i++)
            {
                var legendItem = legendItems[i];

                // Draw legend color box
                Rectangle colorBox = new()
                {
                    Width = 20,
                    Height = 20,
                    Fill = legendItem.Color
                };
                Canvas.SetLeft(colorBox, 0);
                Canvas.SetTop(colorBox, -110 + i * 25 + offset);
                chartCanvas.Children.Add(colorBox);

                // Draw legend text
                TextBlock legendText = new()
                {
                    Text = legendItem.Text,
                    Foreground = Brushes.White,
                    FontSize = 20
                };
                legendText.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));

                Canvas.SetLeft(legendText, 25);
                Canvas.SetTop(legendText, -115 + i * 25 + offset);
                chartCanvas.Children.Add(legendText);
            }
        }

        private void UpdateSeriesValues()
        {
            // Calculate the new values for each series
            TimeSpan totalTime = breakTotal + pausedTotal + productiveTotal;
            TimeSpan distractedTime = distractedTotal;
            TimeSpan breakTime = breakTotal;
            TimeSpan pausedTime = pausedTotal;
            TimeSpan productiveTime = totalTime - distractedTime - breakTime - pausedTime;

            // Convert TimeSpan to formatted strings
            string distractedTimeFormatted = FormatTimeSpan(distractedTime);
            string breakTimeFormatted = FormatTimeSpan(breakTime);
            string pausedTimeFormatted = FormatTimeSpan(pausedTime);
            string productiveTimeFormatted = FormatTimeSpan(productiveTime);

            // Update the series values
            Series =
                [
                    new PieSeries<double> {
                            Name = $"Distractions:",
                            Values = distractedTime.TotalSeconds > 1 ? [distractedTime.TotalMinutes] : new List<double>(),
                            MaxRadialColumnWidth = 35,
                            Fill = new SolidColorPaint(SKColors.Red),
                            ToolTipLabelFormatter = value => $"{distractedTimeFormatted} minutes"
                        },
                        new PieSeries<double> {
                            Name = $"Breaks:",
                            Values = breakTime.TotalSeconds > 1 ? [breakTime.TotalMinutes] : new List<double>(),
                            MaxRadialColumnWidth = 35,
                            Fill = new SolidColorPaint(SKColors.Blue),
                            ToolTipLabelFormatter = value => $"{breakTimeFormatted} minutes"
                        },
                        new PieSeries<double> {
                            Name = $"Manually paused:",
                            Values = pausedTime.TotalSeconds > 1 ? [pausedTime.TotalMinutes] : new List<double>(),
                            MaxRadialColumnWidth = 35,
                            Fill = new SolidColorPaint(SKColors.LightBlue),
                            ToolTipLabelFormatter = value => $"{pausedTimeFormatted} minutes"
                        },
                        new PieSeries<double> {
                            Name = $"Productive:",
                            Values = productiveTime.TotalSeconds > 1 ? [productiveTime.TotalMinutes] : new List<double>(),
                            MaxRadialColumnWidth = 35,
                            Fill = new SolidColorPaint(SKColors.Green),
                            ToolTipLabelFormatter = value => $"{productiveTimeFormatted} minutes"
                        }
                ];

            CenterImageSource = new BitmapImage(new Uri($"pack://application:,,,/Assets/MascottAnimation/Up/{final_stage}/frame0.png"));

            // Notify the UI to update the chart
            DataContext = null;
            DataContext = this;
        }

        // Helper method to format TimeSpan as "mm:ss"
        private static string FormatTimeSpan(TimeSpan timeSpan)
        {
            return $"{(int)timeSpan.TotalMinutes}:{timeSpan.Seconds:D2}";
        }

        public class SessionData
        {
            public required Dictionary<int, string> CycleStartTimes { get; set; }
            public required string SessionEndTime { get; set; }
            public required List<ProcessInfo> ProcessInfoList { get; set; }
            public required List<BreakInfo> BreakInfoList { get; set; }
            public required int FinalDistractionStage { get; set; }
        }

        public class ProcessInfo
        {
            public required string ProcessName { get; set; }
            public required string StartTime { get; set; }
            public required string EndTime { get; set; }
            public required int Cycle { get; set; }
        }

        public class BreakInfo
        {
            public required string StartTime { get; set; }
            public required string EndTime { get; set; }
            public required bool IsBreak { get; set; }
            public required int Cycle { get; set; }
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new Distractions());
        }

        private void Button_Click_3(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new ProgramsList());
        }

        private void Button_Click_4(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new Overview());
        }

        private void Button_Click_5(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new Statistics());
        }

        private void Button_Click_6(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new Abonnement());
        }

        private void Button_Click_7(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new Contact());
        }

        private void Button_Click_8(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new Settings());
        }

        private void BurgerMenuButton_Click(object sender, RoutedEventArgs e)
        {
            MenuPanel.Visibility = MenuPanel.Visibility == Visibility.Visible ? Visibility.Collapsed : Visibility.Visible;
        }
    }
}
