﻿using System.Diagnostics;
using System.IO;
using System.Management;
using System.Runtime;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Navigation;
using Newtonsoft.Json;

namespace InnoLabProjektDektopApp.Services
{
    public class ProcessInfo(string processName, DateTime startTime, DateTime endTime, int cycle)
    {
        public string ProcessName { get; set; } = processName;
        public DateTime StartTime { get; set; } = startTime;
        public DateTime EndTime { get; set; } = endTime;
        public int Cycle { get; set; } = cycle;
    }

    public class BreakInfo(DateTime startTime, DateTime endTime, bool isBreak, int cycle)
    {
        public DateTime StartTime { get; set; } = startTime;
        public DateTime EndTime { get; set; } = endTime;
        public bool IsBreak { get; set; } = isBreak;
        public int Cycle { get; set; } = cycle;
    }

    class ProcessMonitor
    {
        private static List<string> blockedProcesses = [];

        private readonly Dictionary<string, DateTime> processStartTimes = [];
        private readonly List<ProcessInfo> processInfoList = [];
        public bool isMonitoring = false;

        private readonly Dictionary<int, DateTime> cycleStartTimeList = [];

        private readonly List<BreakInfo> breakInfoList = [];
        public bool IsBreak { get; private set;}
        public bool IsPause { get; private set; }
        private DateTime breakStartTime;
        public int currentCycle = 1;
        public string lastSessionInfoFilePath;

        public event EventHandler MonitorCycleStarted;
        public event EventHandler MonitorPaused;
        public event EventHandler MonitorResumed;
        public event EventHandler MonitorBreakStarted;
        public event EventHandler MonitorSessionFinished;
        private ManagementEventWatcher watcher;

        public ProcessMonitor()
        {
            IsBreak = false;
            IsPause = false;
            blockedProcesses = LoadBlockedProcesses();
            var query = new WqlEventQuery("__InstanceCreationEvent", new TimeSpan(0, 0, 1),
                "TargetInstance isa \"Win32_Process\"");
            watcher = new ManagementEventWatcher { Query = query };
            watcher.EventArrived += (sender, e) => HandleProcessEvent(e.NewEvent);
        }

        public async void StartMonitoring()
        {
            if (isMonitoring || IsBreak || IsPause)
            {
                return;
            }
            if (cycleStartTimeList.Count == 0)
            {
                MonitorCycleStarted?.Invoke(this, EventArgs.Empty);
                cycleStartTimeList[1] = DateTime.Now;
            }
            Debug.WriteLine("Started Process Monitoring");

            isMonitoring = true;
            await Task.Run(() => watcher.Start());

            // check if any distracting processes are already running
            var processes = GetRunningProcessList();
            foreach (var process in processes)
            {
                if (IsProcessOnBlockedList(process.Key))
                {
                    ShowDistractionWarning(process.Key);
                }
            }
        }

        private void HandleProcessEvent(ManagementBaseObject e)
        {
            if(!isMonitoring)
            {
                return;
            }
            Task.Run(() => HandleNewProcess(e));

        }

        public void StopMonitoring()
        {
            if (!isMonitoring)
            {
                return;
            }
            watcher.Stop();
            Debug.WriteLine("Stopping Process Monitoring");
            foreach(var process in processStartTimes)
            {
                SaveTrackedData(process.Key, process.Value);
            }
            processStartTimes.Clear();
            CalculateCurrentDistractionStage(DateTime.Now);
            isMonitoring = false;
        }

        public void StartBreak()
        {
            if (IsBreak)
            {
                return;
            }
            IsBreak = true;
            breakStartTime = DateTime.Now;
            StopMonitoring();
            MonitorBreakStarted?.Invoke(this, EventArgs.Empty);
        }

        public void EndBreak()
        {
            if (!IsBreak)
            {
                return;
            }
            breakInfoList.Add(new BreakInfo(breakStartTime, DateTime.Now, IsBreak, currentCycle));
            IsBreak = false;
            currentCycle++;
            cycleStartTimeList[currentCycle] = DateTime.Now;
            StartMonitoring();
            MonitorCycleStarted?.Invoke(this, EventArgs.Empty);
        }

        public void StartPause()
        {
            if (IsPause)
            {
                return;
            }
            IsPause = true;
            breakStartTime = DateTime.Now;
            StopMonitoring();
            MonitorPaused?.Invoke(this, EventArgs.Empty);
        }

        public void EndPause()
        {
            if (!IsPause)
            {
                return;
            }
            breakInfoList.Add(new BreakInfo(breakStartTime, DateTime.Now, IsBreak, currentCycle));
            IsPause = false;
            StartMonitoring();
            MonitorResumed?.Invoke(this, EventArgs.Empty);
        }

        public string FinishSession()
        {
            StopMonitoring();
            if (IsBreak)
            {
                breakInfoList.Add(new BreakInfo(breakStartTime, DateTime.Now, IsBreak, currentCycle));
                IsBreak = false;
            }
            
            // Create a session info object
            var sessionInfo = new
            {
                CycleStartTimes = cycleStartTimeList,
                SessionEndTime = DateTime.Now,
                ProcessInfoList = processInfoList,
                BreakInfoList = breakInfoList,
                FinalDistractionStage = CalculateCurrentDistractionStage(DateTime.Now)
            };

            // the file will be saved in the following folder:
            // "Assets\\sessionData\\YEAR\\MONTH\\DAY\\sessionnumber.json"
            // the session number should be the number of files in the folder + 1

            // Save the session info to a file
            var sessionFolderPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Assets", "sessionData", cycleStartTimeList[1].Year.ToString(), cycleStartTimeList[1].Month.ToString(), cycleStartTimeList[1].Day.ToString());
            Directory.CreateDirectory(sessionFolderPath);
            var sessionNumber = Directory.GetFiles(sessionFolderPath).Length + 1;
            var sessionInfoFilePath = Path.Combine(sessionFolderPath, $"{sessionNumber}.json");
            var sessionInfoJson = JsonConvert.SerializeObject(sessionInfo, Formatting.Indented);
            File.WriteAllText(sessionInfoFilePath, sessionInfoJson);

            // Raise event
            lastSessionInfoFilePath = sessionInfoFilePath;
            MonitorSessionFinished?.Invoke(this, EventArgs.Empty);

            // Reset the processInfoList and breakInfoList
            processInfoList.Clear();
            breakInfoList.Clear();
            processStartTimes.Clear();
            cycleStartTimeList.Clear();
            currentCycle = 1;
            return sessionInfoFilePath;
        }

        /*
        public static int tmpStage = 4;

        public static int CalculateCurrentDistractionStageTmp(DateTime endTime) {
            return tmpStage;
        }*/

        public int CalculateCurrentDistractionStage(DateTime endTime)
        {
            if (cycleStartTimeList.Count == 0) return 4;
            TimeSpan totalDistractionTime = CalculateTotalDistractionTime();
            TimeSpan totalBreakTime = breakInfoList.Aggregate(TimeSpan.Zero, (sum, interval) => sum + (interval.EndTime - interval.StartTime));
            Debug.WriteLine($"Total distraction time: {totalDistractionTime}");

            // Adds a dummy value to the distractedMinutes and totalMinutes so the stage starts in the middle
            double dummyTimeInMinutes = 8;
            double totalMinutes = (endTime - cycleStartTimeList[1]).TotalMinutes - totalBreakTime.TotalMinutes + 2 * dummyTimeInMinutes;
            double distractedMinutes = totalDistractionTime.TotalMinutes + dummyTimeInMinutes;
            double distractionFraction = (distractedMinutes / totalMinutes);
            int stage = (int)Math.Ceiling(distractionFraction * 7);
            Debug.WriteLine($"Distraction stage: {stage}");
            return stage;
        }


        public static List<(string Name, string MainWindowTitle)> GetWindowedProcesses()
        {
            List<string> ignoredProcessesList = ["coflow", "systemsettings", "devenv", "setup"];
            var throwaway = Process.GetProcesses();
            var processes = Process.GetProcesses()
                                   .Where(p => !string.IsNullOrWhiteSpace(p.MainWindowTitle))
                                   .Where(p => !ignoredProcessesList.Contains(p.ProcessName.ToLower()))
                                   .Select(p => (p.ProcessName, p.MainWindowTitle))
                                   .ToList();
            return processes;
        }

        private void HandleNewProcess(ManagementBaseObject e)
        {
            var processName = Path.GetFileNameWithoutExtension((string)((ManagementBaseObject)e["TargetInstance"])["Name"]);
            ShowDistractionWarning(processName);
        }

        private void ShowDistractionWarning(string processName)
        {
            // Check if the process is already being tracked
            if (processStartTimes.TryGetValue(processName, out _)) return;
            if (IsProcessOnBlockedList(processName))
            {
                processStartTimes[processName] = DateTime.Now; // Add the process to the list of tracked to prevent multiple warnings
                var result = MessageBox.Show($"Are you sure you want to get distracted by {processName}?", "Distracting Process Detected",
                MessageBoxButton.YesNo,
                MessageBoxImage.Question,
                MessageBoxResult.No,
                MessageBoxOptions.DefaultDesktopOnly
                );

                if (result == MessageBoxResult.No)
                {
                    KillProcessesByName(processName);
                    processStartTimes.Remove(processName);
                }
                else
                {
                    // Save the start time of the process so we can calculate the time it has been running without waiting for the process to end
                    processStartTimes[processName] = DateTime.Now;
                    TrackProcess(processName, DateTime.Now);
                }
            }
        }

        private static void KillProcessesByName(String processName)
        {
            var runningProcesses = GetRunningProcessList();
            if (!runningProcesses.TryGetValue(processName, out List<Process>? processes) || processes == null)
            {
                return; // Return early if the process name is not found or processes is null
            }

            Debug.WriteLine("Killing {0} processes with the Name {1}", processes.Count, processName);
            foreach (var process in processes)
            {
                process.Kill();
            }
        }

        private static Dictionary<string, List<Process>> GetRunningProcessList()
        {
            var processlist = Process.GetProcesses();
            var processDictionary = new Dictionary<string, List<Process>>();

            foreach (var process in processlist)
            {
                var processName = process.ProcessName;

                if (!processDictionary.TryGetValue(processName, out List<Process>? value))
                {
                    value = [];
                    processDictionary[processName] = value;
                }

                value.Add(process);
            }

            return processDictionary;
        }

        private static bool IsProcessOnBlockedList(string processName)
        {
            return blockedProcesses.Contains(processName.ToLower());
        }

        private async void TrackProcess(string processName, DateTime startTime)
        {
            Debug.WriteLine($"Start monitoring the {processName} process");

            Process? process = GetRunningProcessList().TryGetValue(processName, out List<Process>? processes) ? processes[0] : null;

            if (process == null)
            {
                Debug.WriteLine("Process exited before we could track it.");
                return;
            }

            // Token to handle task cancellation
            using var cts = new CancellationTokenSource();
            var token = cts.Token;

            // Task to wait for the process to exit
            var waitForExitTask = Task.Run(() =>
            {
                process.WaitForExit();
            }, token);

            // Task to monitor the isMonitoring flag
            var monitorTask = Task.Run(async () =>
            {
                while (isMonitoring)
                {
                    await Task.Delay(100); // Check every 100ms
                }
            }, token);

            try
            {
                // Await the completion of either the waitForExitTask or the monitorTask
                await Task.WhenAny(waitForExitTask, monitorTask);
                cts.Cancel();
                SaveTrackedData(processName, startTime);
            }
            catch (OperationCanceledException)
            {
                Debug.WriteLine("Process monitoring was canceled.");
            }
        }

        private void SaveTrackedData(String processName, DateTime startTime)
        {
            processStartTimes.Remove(processName);
            var endTime = DateTime.Now;
            processInfoList.Add(new ProcessInfo(processName, startTime, endTime, currentCycle));
            Debug.WriteLine("Process monitoring complete.");
            Debug.WriteLine("The Process has been running for {0} seconds", endTime - startTime);
        }


        public TimeSpan CalculateTotalDistractionTime()
        {
            var tempProcessInfoList = new List<ProcessInfo>();

            // Add the earliest start time entry from processStartTimes to tempProcessInfoList
            if (processStartTimes.Count > 0)
            {
                var earliestProcess = processStartTimes.OrderBy(p => p.Value).First();
                tempProcessInfoList.Add(new ProcessInfo(earliestProcess.Key, earliestProcess.Value, DateTime.Now, currentCycle));
            }

            // Add all entries from processInfoList to tempProcessInfoList
            tempProcessInfoList.AddRange(processInfoList);

            var intervals = tempProcessInfoList
                .OrderBy(p => p.StartTime)
                .Select(p => (p.StartTime, p.EndTime))
                .ToList();

            if (intervals.Count == 0)
                return TimeSpan.Zero;

            var mergedIntervals = MergeIntervals(intervals);
            return SubtractBreaksFromDistractionTime(mergedIntervals, [.. breakInfoList.Select(b => (b.StartTime, b.EndTime))]);
        }

        public static List<(DateTime Start, DateTime End)> MergeIntervals(List<(DateTime Start, DateTime End)> intervals)
        {
            // return a list that includes DateTime.now to DateTime.now to prevent the stage from being 0
            if (intervals.Count == 0)
            {
                return [(DateTime.Now, DateTime.Now)];
            }
            var mergedIntervals = new List<(DateTime Start, DateTime End)> { 
                intervals[0]
            };
            foreach (var interval in intervals.Skip(1))
            {
                var last = mergedIntervals.Last();
                // If the current interval overlaps with the last merged interval, merge them
                if (interval.Start <= last.End)
                {
                    mergedIntervals[^1] = (last.Start, new DateTime(Math.Max(last.End.Ticks, interval.End.Ticks)));
                }
                else
                {
                    // Otherwise, add the current interval as a new merged interval
                    mergedIntervals.Add(interval);
                }
            }
            return mergedIntervals;
        }

        public static TimeSpan SubtractBreaksFromDistractionTime(
            List<(DateTime Start, DateTime End)> mergedDistractionIntervals, 
            List<(DateTime Start, DateTime End)> breakIntervals)
        {
            foreach (var breakInfo in breakIntervals)
            {
                mergedDistractionIntervals = [.. mergedDistractionIntervals.SelectMany(interval =>
                {
                    // If the interval ends before the break starts or starts after the break ends, keep the interval as is
                    if (interval.End <= breakInfo.Start || interval.Start >= breakInfo.End)
                    {
                        return [interval];
                    }
                    // If the interval starts before the break and ends after the break, split the interval into two
                    else if (interval.Start < breakInfo.Start && interval.End > breakInfo.End)
                    {
                        return
                        [
                            (interval.Start, breakInfo.Start),
                            (breakInfo.End, interval.End)
                        ];
                    }
                    // If the interval starts before the break and ends during the break, adjust the end time to the start of the break
                    else if (interval.Start < breakInfo.Start)
                    {
                        return new List<(DateTime Start, DateTime End)> { (interval.Start, breakInfo.Start) };
                    }
                    // If the interval starts during the break and ends after the break, adjust the start time to the end of the break
                    else if (interval.End > breakInfo.End)
                    {
                        return [(breakInfo.End, interval.End)];
                    }
                    // If the interval is completely within the break, remove the interval
                    else
                    {
                        return [];
                    }
                })];
            }
            return mergedDistractionIntervals.Aggregate(TimeSpan.Zero, (sum, interval) => sum + (interval.End - interval.Start));
        }


        private static List<string> LoadBlockedProcesses()
        {
            var blockedProcessesFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Assets\\distractingPrograms.json");
            var fileContent = File.ReadAllText(blockedProcessesFilePath);
            var programs = JsonConvert.DeserializeObject<dynamic>(fileContent);
            var blockedProcesses = new List<string>();

            if (programs?.Programs != null)
            {
                foreach (var program in programs.Programs)
                {
                    if (program.distracting == true)
                    {
                        string processName = program.processName;
                        blockedProcesses.Add(processName.ToLower());
                    }
                }
            }

            return blockedProcesses;
        }
    }
}
